@@ -9,8 +9,8 @@ use clap::{crate_version, Arg, ArgAction, ArgMatches, Command};
 use std::fs::File;
 use std::io::{stdin, stdout, BufReader, BufWriter, Read, Stdout, Write};
 use uucore::display::Quotable;
-use uucore::error::{FromIo, UResult, USimpleError};
-use uucore::{format_usage, help_about, help_usage, show_warning};
+use uucore::error::{FromIo, UResult, USimpleError, UUsageError};
+use uucore::{format_usage, help_about, help_usage};
 
 use linebreak::break_lines;
 use parasplit::ParagraphStream;
@@ -21,6 +21,10 @@ mod parasplit;
 const ABOUT: &str = help_about!("fmt.md");
 const USAGE: &str = help_usage!("fmt.md");
 const MAX_WIDTH: usize = 2500;
+const DEFAULT_GOAL: usize = 70;
+const DEFAULT_WIDTH: usize = 75;
+// by default, goal is 93% of width
+const DEFAULT_GOAL_TO_WIDTH_RATIO: usize = 93;
 
 mod options {
     pub const CROWN_MARGIN: &str = "crown-margin";
@@ -36,13 +40,11 @@ mod options {
     pub const GOAL: &str = "goal";
     pub const QUICK: &str = "quick";
     pub const TAB_WIDTH: &str = "tab-width";
-    pub const FILES: &str = "files";
+    pub const FILES_OR_WIDTH: &str = "files";
 }
 
-// by default, goal is 93% of width
-const DEFAULT_GOAL_TO_WIDTH_RATIO: usize = 93;
-
 pub type FileOrStdReader = BufReader<Box<dyn Read + 'static>>;
+
 pub struct FmtOptions {
     crown: bool,
     tagged: bool,
@@ -85,25 +87,44 @@ impl FmtOptions {
             .get_one::<String>(options::SKIP_PREFIX)
             .map(String::from);
 
-        let width_opt = matches.get_one::<usize>(options::WIDTH);
-        let goal_opt = matches.get_one::<usize>(options::GOAL);
+        let width_opt = extract_width(matches)?;
+        let goal_opt_str = matches.get_one::<String>(options::GOAL);
+        let goal_opt = if let Some(goal_str) = goal_opt_str {
+            match goal_str.parse::<usize>() {
+                Ok(goal) => Some(goal),
+                Err(_) => {
+                    return Err(USimpleError::new(
+                        1,
+                        format!("invalid goal: {}", goal_str.quote()),
+                    ));
+                }
+            }
+        } else {
+            None
+        };
+
         let (width, goal) = match (width_opt, goal_opt) {
-            (Some(&w), Some(&g)) => {
+            (Some(w), Some(g)) => {
                 if g > w {
                     return Err(USimpleError::new(1, "GOAL cannot be greater than WIDTH."));
                 }
                 (w, g)
             }
-            (Some(&w), None) => {
-                let g = (w * DEFAULT_GOAL_TO_WIDTH_RATIO / 100).min(w - 3);
+            (Some(w), None) => {
+                // Only allow a goal of zero if the width is set to be zero
+                let g = (w * DEFAULT_GOAL_TO_WIDTH_RATIO / 100).max(if w == 0 { 0 } else { 1 });
                 (w, g)
             }
-            (None, Some(&g)) => {
+            (None, Some(g)) => {
+                if g > DEFAULT_WIDTH {
+                    return Err(USimpleError::new(1, "GOAL cannot be greater than WIDTH."));
+                }
                 let w = (g * 100 / DEFAULT_GOAL_TO_WIDTH_RATIO).max(g + 3);
                 (w, g)
             }
-            (None, None) => (75, 70),
+            (None, None) => (DEFAULT_WIDTH, DEFAULT_GOAL),
         };
+        debug_assert!(width >= goal, "GOAL {goal} should not be greater than WIDTH {width} when given {width_opt:?} and {goal_opt:?}.");
 
         if width > MAX_WIDTH {
             return Err(USimpleError::new(
@@ -163,16 +184,14 @@ fn process_file(
     fmt_opts: &FmtOptions,
     ostream: &mut BufWriter<Stdout>,
 ) -> UResult<()> {
-    let mut fp = match file_name {
-        "-" => BufReader::new(Box::new(stdin()) as Box<dyn Read + 'static>),
-        _ => match File::open(file_name) {
-            Ok(f) => BufReader::new(Box::new(f) as Box<dyn Read + 'static>),
-            Err(e) => {
-                show_warning!("{}: {}", file_name.maybe_quote(), e);
-                return Ok(());
-            }
-        },
-    };
+    let mut fp = BufReader::new(match file_name {
+        "-" => Box::new(stdin()) as Box<dyn Read + 'static>,
+        _ => {
+            let f = File::open(file_name)
+                .map_err_context(|| format!("cannot open {} for reading", file_name.quote()))?;
+            Box::new(f) as Box<dyn Read + 'static>
+        }
+    });
 
     let p_stream = ParagraphStream::new(fmt_opts, &mut fp);
     for para_result in p_stream {
@@ -198,14 +217,97 @@ fn process_file(
     Ok(())
 }
 
+/// Extract the file names from the positional arguments, ignoring any negative width in the first
+/// position.
+///
+/// # Returns
+/// A `UResult<()>` with the file names, or an error if one of the file names could not be parsed
+/// (e.g., it is given as a negative number not in the first argument and not after a --
+fn extract_files(matches: &ArgMatches) -> UResult<Vec<String>> {
+    let in_first_pos = matches
+        .index_of(options::FILES_OR_WIDTH)
+        .is_some_and(|x| x == 1);
+    let is_neg = |s: &str| s.parse::<isize>().is_ok_and(|w| w < 0);
+
+    let files: UResult<Vec<String>> = matches
+        .get_many::<String>(options::FILES_OR_WIDTH)
+        .into_iter()
+        .flatten()
+        .enumerate()
+        .filter_map(|(i, x)| {
+            if is_neg(x) {
+                if in_first_pos && i == 0 {
+                    None
+                } else {
+                    let first_num = x.chars().nth(1).expect("a negative number should be at least two characters long");
+                    Some(Err(
+                        UUsageError::new(1, format!("invalid option -- {}; -WIDTH is recognized only when it is the first\noption; use -w N instead", first_num))
+                    ))
+                }
+            } else {
+                Some(Ok(x.clone()))
+            }
+        })
+        .collect();
+
+    if files.as_ref().is_ok_and(|f| f.is_empty()) {
+        Ok(vec!["-".into()])
+    } else {
+        files
+    }
+}
+
+fn extract_width(matches: &ArgMatches) -> UResult<Option<usize>> {
+    let width_opt = matches.get_one::<String>(options::WIDTH);
+    if let Some(width_str) = width_opt {
+        if let Ok(width) = width_str.parse::<usize>() {
+            return Ok(Some(width));
+        } else {
+            return Err(USimpleError::new(
+                1,
+                format!("invalid width: {}", width_str.quote()),
+            ));
+        }
+    }
+
+    if let Some(1) = matches.index_of(options::FILES_OR_WIDTH) {
+        let width_arg = matches.get_one::<String>(options::FILES_OR_WIDTH).unwrap();
+        if let Some(num) = width_arg.strip_prefix('-') {
+            Ok(num.parse::<usize>().ok())
+        } else {
+            // will be treated as a file name
+            Ok(None)
+        }
+    } else {
+        Ok(None)
+    }
+}
+
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().try_get_matches_from(args)?;
+    let args: Vec<_> = args.collect();
 
-    let files: Vec<String> = matches
-        .get_many::<String>(options::FILES)
-        .map(|v| v.map(ToString::to_string).collect())
-        .unwrap_or(vec!["-".into()]);
+    // Warn the user if it looks like we're trying to pass a number in the first
+    // argument with non-numeric characters
+    if let Some(first_arg) = args.get(1) {
+        let first_arg = first_arg.to_string_lossy();
+        let malformed_number = first_arg.starts_with('-')
+            && first_arg.chars().nth(1).is_some_and(|c| c.is_ascii_digit())
+            && first_arg.chars().skip(2).any(|c| !c.is_ascii_digit());
+        if malformed_number {
+            return Err(USimpleError::new(
+                1,
+                format!(
+                    "invalid width: {}",
+                    first_arg.strip_prefix('-').unwrap().quote()
+                ),
+            ));
+        }
+    }
+
+    let matches = uu_app().try_get_matches_from(&args)?;
+
+    let files = extract_files(&matches)?;
 
     let fmt_opts = FmtOptions::from_matches(&matches)?;
 
@@ -224,6 +326,7 @@ pub fn uu_app() -> Command {
         .about(ABOUT)
         .override_usage(format_usage(USAGE))
         .infer_long_args(true)
+        .args_override_self(true)
         .arg(
             Arg::new(options::CROWN_MARGIN)
                 .short('c')
@@ -323,17 +426,17 @@ pub fn uu_app() -> Command {
             Arg::new(options::WIDTH)
                 .short('w')
                 .long("width")
-                .help("Fill output lines up to a maximum of WIDTH columns, default 75.")
-                .value_name("WIDTH")
-                .value_parser(clap::value_parser!(usize)),
+                .help("Fill output lines up to a maximum of WIDTH columns, default 75. This can be specified as a negative number in the first argument.")
+                // We must accept invalid values if they are overridden later. This is not supported by clap, so accept all strings instead.
+                .value_name("WIDTH"),
         )
         .arg(
             Arg::new(options::GOAL)
                 .short('g')
                 .long("goal")
-                .help("Goal width, default of 93% of WIDTH. Must be less than WIDTH.")
-                .value_name("GOAL")
-                .value_parser(clap::value_parser!(usize)),
+                .help("Goal width, default of 93% of WIDTH. Must be less than or equal to WIDTH.")
+                // We must accept invalid values if they are overridden later. This is not supported by clap, so accept all strings instead.
+                .value_name("GOAL"),
         )
         .arg(
             Arg::new(options::QUICK)
@@ -357,8 +460,64 @@ pub fn uu_app() -> Command {
                 .value_name("TABWIDTH"),
         )
         .arg(
-            Arg::new(options::FILES)
+            Arg::new(options::FILES_OR_WIDTH)
                 .action(ArgAction::Append)
-                .value_hint(clap::ValueHint::FilePath),
+                .value_name("FILES")
+                .value_hint(clap::ValueHint::FilePath)
+                .allow_negative_numbers(true),
         )
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::uu_app;
+    use crate::{extract_files, extract_width};
+
+    #[test]
+    fn parse_negative_width() {
+        let matches = uu_app()
+            .try_get_matches_from(vec!["fmt", "-3", "some-file"])
+            .unwrap();
+
+        assert_eq!(extract_files(&matches).unwrap(), vec!["some-file"]);
+        assert_eq!(extract_width(&matches).ok(), Some(Some(3)));
+    }
+
+    #[test]
+    fn parse_width_as_arg() {
+        let matches = uu_app()
+            .try_get_matches_from(vec!["fmt", "-w3", "some-file"])
+            .unwrap();
+
+        assert_eq!(extract_files(&matches).unwrap(), vec!["some-file"]);
+        assert_eq!(extract_width(&matches).ok(), Some(Some(3)));
+    }
+
+    #[test]
+    fn parse_no_args() {
+        let matches = uu_app().try_get_matches_from(vec!["fmt"]).unwrap();
+
+        assert_eq!(extract_files(&matches).unwrap(), vec!["-"]);
+        assert_eq!(extract_width(&matches).ok(), Some(None));
+    }
+
+    #[test]
+    fn parse_just_file_name() {
+        let matches = uu_app()
+            .try_get_matches_from(vec!["fmt", "some-file"])
+            .unwrap();
+
+        assert_eq!(extract_files(&matches).unwrap(), vec!["some-file"]);
+        assert_eq!(extract_width(&matches).ok(), Some(None));
+    }
+
+    #[test]
+    fn parse_with_both_widths_positional_first() {
+        let matches = uu_app()
+            .try_get_matches_from(vec!["fmt", "-10", "-w3", "some-file"])
+            .unwrap();
+
+        assert_eq!(extract_files(&matches).unwrap(), vec!["some-file"]);
+        assert_eq!(extract_width(&matches).ok(), Some(Some(3)));
+    }
+}
